/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('entity-content-revisions', function () {

  juju.components.EntityContentRevisions = React.createClass({
    displayName: 'EntityContentRevisions',

    /**
      Get the current state of the revisions panel.
       @method getInitialState
      @returns {String} The current state.
    */
    getInitialState: function getInitialState() {
      // Setting a default state object.
      var state = {
        showMore: false
      };
      return state;
    },

    /**
      Handle clicks on accordion. Toggles the concealed class on the list node
      which hides all the list items apart from the first.
       @method _handleAccordionClick
    */
    _handleAccordionClick: function _handleAccordionClick(conceal) {
      this.setState({ showMore: conceal });
    },

    /**
      Format ISO date/time to human readable format (2015-09-05).
       @method _formatDate
      @param {String} iso A date to be parsed.
    */
    _formatDate: function _formatDate(iso) {
      var date = new Date(Date.parse(iso));
      date = date.getUTCFullYear() + '-' + ('0' + (date.getUTCMonth() + 1)).slice(-2) + '-' + ('0' + date.getUTCDate()).slice(-2);
      return date;
    },

    /**
      Generate the classname to apply based on the state.
       @method _generateRevisions
      @return {Object} The revisions markup.
    */
    _generateClassName: function _generateClassName() {
      return classNames('revisions__list', {
        'list--concealed': !this.state.showMore
      });
    },

    /**
      Generate the list of revisions.
       @method _generateRevisions
      @return {Object} The revisions markup.
    */
    _generateRevisions: function _generateRevisions() {
      var _this = this;

      var components = [];
      var revisions = this.props.revisions;
      revisions.forEach(function (revision) {
        var date = _this._formatDate(revision.date);
        components.push(React.createElement(
          'li',
          { className: 'revisions__list-item list-item', key: revision.revno },
          React.createElement(
            'p',
            { className: 'revisions__list-meta smaller' },
            'by ',
            revision.authors[0].name,
            React.createElement(
              'span',
              { className: 'revisions__list-meta-date' },
              date
            )
          ),
          React.createElement(
            'p',
            { className: 'revisions__list-message' },
            revision.message
          )
        ));
      }, this);
      return components;
    },

    render: function render() {
      var revisions = this.props.revisions;
      var revisionsCount = revisions.length;
      return React.createElement(
        'div',
        { className: 'revisions section', id: 'revisions' },
        React.createElement(
          'h3',
          { className: 'section__title' },
          revisionsCount,
          ' Revisions'
        ),
        React.createElement(
          'ol',
          { className: this._generateClassName(), ref: 'list', reversed: true },
          this._generateRevisions(),
          React.createElement(
            'li',
            { className: 'list__controls' },
            React.createElement(
              'a',
              { href: '', className: 'btn__see--more',
                onClick: this._handleAccordionClick.bind(this, true) },
              'See more'
            ),
            React.createElement(
              'a',
              { href: '#revisions', className: 'btn__see--less',
                onClick: this._handleAccordionClick.bind(this, false) },
              'See less'
            )
          )
        )
      );
    }
  });
}, '0.1.0', { requires: [] });
