/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('generic-button', function () {

  juju.components.GenericButton = React.createClass({
    displayName: 'GenericButton',

    propTypes: {
      icon: React.PropTypes.string,
      submit: React.PropTypes.bool,
      title: React.PropTypes.string
    },

    /**
      Returns the classes for the button based on the provided props.
      @method _generateClasses
      @returns {String} The collection of class names.
    */
    _generateClasses: function _generateClasses() {
      return classNames('generic-button', this.props.type ? 'generic-button--type-' + this.props.type : '', {
        'generic-button--disabled': this.props.disabled
      });
    },

    /**
      Call the action if not disabled.
       @method _handleClick
      @param {Object} e The click event.
    */
    _handleClick: function _handleClick(e) {
      // Don't bubble the click the parent.
      e.stopPropagation();
      if (!this.props.disabled) {
        this.props.action();
      }
    },

    /**
      Generate the label or icon.
       @method _generateContent
    */
    _generateContent: function _generateContent() {
      var title = this.props.title;
      var icon = this.props.icon;
      if (title) {
        return title;
      } else if (icon) {
        return React.createElement(juju.components.SvgIcon, { name: icon,
          size: '16' });
      }
    },

    render: function render() {
      return React.createElement(
        'button',
        { className: this._generateClasses(),
          onClick: this._handleClick,
          type: this.props.submit ? 'submit' : 'button' },
        this._generateContent()
      );
    }
  });
}, '0.1.0', { requires: ['svg-icon'] });
