/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('inspector-expose', function () {

  juju.components.InspectorExpose = React.createClass({
    displayName: 'InspectorExpose',

    /**
      The callable to be passed to the unit items for navigating to the unit
      details.
       @method _unitItemAction
      @param {Object} e The click event.
    */
    _unitItemAction: function _unitItemAction(e) {
      var unitId = e.currentTarget.getAttribute('data-id').split('/')[1];
      this.props.changeState({
        sectionA: {
          component: 'inspector',
          metadata: {
            id: this.props.service.get('id'),
            unit: unitId,
            activeComponent: 'unit'
          }
        }
      });
    },

    /**
      Generate a list of units for the service
       @method _generateUnits
      @returns {Array} A list of units.
    */
    _generateUnits: function _generateUnits() {
      var units = [];
      this.props.units.toArray().forEach(function (unit) {
        units.push(React.createElement(juju.components.InspectorExposeUnit, {
          key: unit.id,
          action: this._unitItemAction,
          unit: unit }));
      }, this);
      return units;
    },

    /**
      Display a list of units if the service is exposed.
       @method _displayUnitList
      @returns {Object} A list of units.
    */
    _displayUnitList: function _displayUnitList() {
      if (!this.props.service.get('exposed')) {
        return;
      }
      return React.createElement(
        'ul',
        { className: 'inspector-expose__units' },
        this._generateUnits()
      );
    },

    /**
      Expose the service when toggled.
       @method _handleExposeChange
    */
    _handleExposeChange: function _handleExposeChange() {
      var service = this.props.service;
      var serviceId = service.get('id');
      if (service.get('exposed')) {
        this.props.unexposeService(serviceId, this._exposeServiceCallback, {});
      } else {
        this.props.exposeService(serviceId, this._exposeServiceCallback, {});
      }
    },

    /**
      Callback to handle errors when exposing a service.
       @method _exposeServiceCallback
      @param {object} e The expose event
    */
    _exposeServiceCallback: function _exposeServiceCallback(e) {
      if (e.err) {
        console.error(e.err);
        this.props.addNotification({
          title: 'Exposing charm failed',
          message: 'The service' + this.props.service.get('name') + ' failed to expose:' + e.err,
          level: 'error'
        });
      }
    },

    render: function render() {
      var toggle = {
        key: 'expose-toggle'
      };
      return React.createElement(
        'div',
        { className: 'inspector-expose' },
        React.createElement(
          'div',
          { className: 'inspector-expose__control' },
          React.createElement(juju.components.BooleanConfig, {
            key: toggle.key,
            ref: toggle.key,
            option: toggle,
            onChange: this._handleExposeChange,
            label: 'Expose service',
            config: this.props.service.get('exposed') })
        ),
        React.createElement(
          'p',
          { className: 'inspector-expose__warning' },
          'Exposing this service may make it publicly accessible from the web'
        ),
        this._displayUnitList()
      );
    }

  });
}, '0.1.0', { requires: ['boolean-config', 'inspector-expose-unit'] });
