/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

chai.config.includeStack = true;
chai.config.truncateThreshold = 0;

describe('LocalInspector', function () {
  var series;

  beforeAll(function (done) {
    // By loading this file it adds the component to the juju components.
    YUI().use('local-inspector', function () {
      done();
    });
  });

  beforeEach(function () {
    series = {
      vivid: { name: 'Vivid Vervet 15.04' },
      wily: { name: 'Wily Werewolf 15.10' }
    };
  });

  it('can display the new service view', function () {
    var file = {
      name: 'apache2.zip',
      size: '2048'
    };
    var services = sinon.spy();
    var uploadLocalCharm = sinon.spy();
    var upgradeServiceUsingLocalCharm = sinon.spy();
    var changeState = sinon.spy();
    var shallowRenderer = jsTestUtils.shallowRender(React.createElement(juju.components.LocalInspector, {
      file: file,
      series: series,
      localType: 'new',
      services: services,
      uploadLocalCharm: uploadLocalCharm,
      upgradeServiceUsingLocalCharm: upgradeServiceUsingLocalCharm,
      changeState: changeState }), true);
    var instance = shallowRenderer.getMountedInstance();
    var output = shallowRenderer.getRenderOutput();
    var buttons = [{
      title: 'Cancel',
      action: instance._close
    }, {
      title: 'Upload',
      action: instance._handleUpload,
      type: 'confirm'
    }];
    var options = output.props.children[1].props.children[1].props.children;
    var expected = React.createElement(
      'div',
      { className: 'inspector-view local-inspector' },
      React.createElement(juju.components.InspectorHeader, {
        backCallback: instance._close,
        title: 'Local charm' }),
      React.createElement(
        'div',
        { className: 'inspector-content local-inspector__section' },
        React.createElement(
          'div',
          { className: 'local-inspector__file' },
          React.createElement(
            'p',
            null,
            'File: ',
            "apache2.zip"
          ),
          React.createElement(
            'p',
            null,
            'Size: ',
            "2.00",
            'kb'
          )
        ),
        React.createElement(
          'ul',
          { className: 'local-inspector__list' },
          React.createElement(
            'li',
            null,
            React.createElement(
              'label',
              null,
              React.createElement('input', { type: 'radio', name: 'action',
                defaultChecked: true,
                onChange: options[0].props.children.props.children[0].props.onChange }),
              'Deploy local'
            )
          ),
          React.createElement(
            'li',
            null,
            React.createElement(
              'label',
              null,
              React.createElement('input', { type: 'radio', name: 'action',
                defaultChecked: false,
                onChange: options[1].props.children.props.children[0].props.onChange }),
              'Upgrade local'
            )
          )
        ),
        React.createElement(
          'div',
          null,
          React.createElement(
            'p',
            null,
            'Choose a series to deploy this charm'
          ),
          React.createElement(
            'select',
            { ref: 'series', defaultValue: 'trusty',
              className: 'local-inspector__series' },
            React.createElement(
              'option',
              { value: 'vivid', key: 'vivid' },
              'Vivid Vervet 15.04'
            ),
            React.createElement(
              'option',
              { value: 'wily', key: 'wily' },
              'Wily Werewolf 15.10'
            )
          )
        )
      ),
      React.createElement(juju.components.ButtonRow, {
        buttons: buttons })
    );
    assert.deepEqual(output, expected);
  });

  it('can display the update service view', function () {
    var file = {
      name: 'apache2.zip',
      size: '2048'
    };
    var serviceStubs = [{ get: sinon.stub() }, { get: sinon.stub() }];
    serviceStubs[0].get.withArgs('id').returns('apache2-2');
    serviceStubs[0].get.withArgs('name').returns('apache2');
    serviceStubs[1].get.withArgs('id').returns('mysql-1');
    serviceStubs[1].get.withArgs('name').returns('mysql');
    var services = { toArray: sinon.stub().returns(serviceStubs) };
    var uploadLocalCharm = sinon.spy();
    var upgradeServiceUsingLocalCharm = sinon.spy();
    var changeState = sinon.spy();
    var shallowRenderer = jsTestUtils.shallowRender(React.createElement(juju.components.LocalInspector, {
      file: file,
      series: series,
      localType: 'update',
      services: services,
      uploadLocalCharm: uploadLocalCharm,
      upgradeServiceUsingLocalCharm: upgradeServiceUsingLocalCharm,
      changeState: changeState }), true);
    var instance = shallowRenderer.getMountedInstance();
    var output = shallowRenderer.getRenderOutput();
    var buttons = [{
      title: 'Cancel',
      action: instance._close
    }, {
      title: 'Upload',
      action: instance._handleUpdate,
      type: 'confirm'
    }];
    var options = output.props.children[1].props.children[1].props.children;
    var expected = React.createElement(
      'div',
      { className: 'inspector-view local-inspector' },
      React.createElement(juju.components.InspectorHeader, {
        backCallback: instance._close,
        title: 'Local charm' }),
      React.createElement(
        'div',
        { className: 'inspector-content local-inspector__section' },
        React.createElement(
          'div',
          { className: 'local-inspector__file' },
          React.createElement(
            'p',
            null,
            'File: ',
            "apache2.zip"
          ),
          React.createElement(
            'p',
            null,
            'Size: ',
            "2.00",
            'kb'
          )
        ),
        React.createElement(
          'ul',
          { className: 'local-inspector__list' },
          React.createElement(
            'li',
            null,
            React.createElement(
              'label',
              null,
              React.createElement('input', { type: 'radio', name: 'action',
                defaultChecked: false,
                onChange: options[0].props.children.props.children[0].props.onChange }),
              'Deploy local'
            )
          ),
          React.createElement(
            'li',
            null,
            React.createElement(
              'label',
              null,
              React.createElement('input', { type: 'radio', name: 'action',
                defaultChecked: true,
                onChange: options[1].props.children.props.children[0].props.onChange }),
              'Upgrade local'
            )
          )
        ),
        React.createElement(
          'ul',
          { className: 'local-inspector__list' },
          React.createElement(
            'li',
            { key: 'apache2-2' },
            React.createElement(
              'label',
              null,
              React.createElement('input', { type: 'checkbox', 'data-id': 'apache2-2',
                ref: 'service-apache2-2' }),
              'apache2'
            )
          ),
          React.createElement(
            'li',
            { key: 'mysql-1' },
            React.createElement(
              'label',
              null,
              React.createElement('input', { type: 'checkbox', 'data-id': 'mysql-1',
                ref: 'service-mysql-1' }),
              'mysql'
            )
          )
        )
      ),
      React.createElement(juju.components.ButtonRow, {
        buttons: buttons })
    );
    assert.deepEqual(output, expected);
  });

  it('can switch between views', function () {
    var file = {
      name: 'apache2.zip',
      size: '2048'
    };
    var serviceStubs = [{ get: sinon.stub() }, { get: sinon.stub() }];
    serviceStubs[0].get.withArgs('id').returns('apache2-2');
    serviceStubs[0].get.withArgs('name').returns('apache2');
    serviceStubs[1].get.withArgs('id').returns('mysql-1');
    serviceStubs[1].get.withArgs('name').returns('mysql');
    var services = { toArray: sinon.stub().returns(serviceStubs) };
    var uploadLocalCharm = sinon.spy();
    var upgradeServiceUsingLocalCharm = sinon.spy();
    var changeState = sinon.spy();
    var shallowRenderer = jsTestUtils.shallowRender(React.createElement(juju.components.LocalInspector, {
      file: file,
      series: series,
      localType: 'new',
      services: services,
      uploadLocalCharm: uploadLocalCharm,
      upgradeServiceUsingLocalCharm: upgradeServiceUsingLocalCharm,
      changeState: changeState }), true);
    var output = shallowRenderer.getRenderOutput();
    var options = output.props.children[1].props.children[1].props.children;
    options[1].props.children.props.children[0].props.onChange();
    output = shallowRenderer.getRenderOutput();
    var expected = React.createElement(
      'div',
      { className: 'inspector-view local-inspector' },
      output.props.children[0],
      React.createElement(
        'div',
        { className: 'inspector-content local-inspector__section' },
        output.props.children[1].props.children[0],
        output.props.children[1].props.children[1],
        React.createElement(
          'ul',
          { className: 'local-inspector__list' },
          React.createElement(
            'li',
            { key: 'apache2-2' },
            React.createElement(
              'label',
              null,
              React.createElement('input', { type: 'checkbox', 'data-id': 'apache2-2',
                ref: 'service-apache2-2' }),
              'apache2'
            )
          ),
          React.createElement(
            'li',
            { key: 'mysql-1' },
            React.createElement(
              'label',
              null,
              React.createElement('input', { type: 'checkbox', 'data-id': 'mysql-1',
                ref: 'service-mysql-1' }),
              'mysql'
            )
          )
        )
      ),
      output.props.children[2]
    );
    assert.deepEqual(output, expected);
  });

  it('can handle deploying a new charm', function () {
    var file = {
      name: 'apache2.zip',
      size: '2048'
    };
    var services = { toArray: sinon.stub().returns([]) };
    var uploadLocalCharm = sinon.spy();
    var upgradeServiceUsingLocalCharm = sinon.spy();
    var changeState = sinon.spy();
    var shallowRenderer = jsTestUtils.shallowRender(React.createElement(juju.components.LocalInspector, {
      file: file,
      series: series,
      localType: 'new',
      services: services,
      uploadLocalCharm: uploadLocalCharm,
      upgradeServiceUsingLocalCharm: upgradeServiceUsingLocalCharm,
      changeState: changeState }), true);
    var instance = shallowRenderer.getMountedInstance();
    var output = shallowRenderer.getRenderOutput();
    instance.refs = { series: { value: 'wily' } };
    output.props.children[2].props.buttons[1].action();
    assert.equal(uploadLocalCharm.callCount, 1);
    assert.equal(uploadLocalCharm.args[0][0], 'wily');
    assert.equal(uploadLocalCharm.args[0][1], file);
  });

  it('can handle updating charms', function () {
    var file = {
      name: 'apache2.zip',
      size: '2048'
    };
    var getById = sinon.stub();
    getById.withArgs('mysql').returns({ id: 'mysql' });
    getById.withArgs('apache2').returns({ id: 'apache2' });
    var services = {
      toArray: sinon.stub().returns([]),
      getById: getById
    };
    var uploadLocalCharm = sinon.spy();
    var upgradeServiceUsingLocalCharm = sinon.spy();
    var changeState = sinon.spy();
    var shallowRenderer = jsTestUtils.shallowRender(React.createElement(juju.components.LocalInspector, {
      file: file,
      series: series,
      localType: 'update',
      services: services,
      uploadLocalCharm: uploadLocalCharm,
      upgradeServiceUsingLocalCharm: upgradeServiceUsingLocalCharm,
      changeState: changeState }), true);
    var instance = shallowRenderer.getMountedInstance();
    var output = shallowRenderer.getRenderOutput();
    instance.refs = {
      'service-mysql': { checked: true },
      'service-django': { checked: false },
      'service-apache2': { checked: true }
    };
    output.props.children[2].props.buttons[1].action();
    assert.equal(upgradeServiceUsingLocalCharm.callCount, 1);
    assert.deepEqual(upgradeServiceUsingLocalCharm.args[0][0], [{ id: 'mysql' }, { id: 'apache2' }]);
    assert.equal(upgradeServiceUsingLocalCharm.args[0][1], file);
    assert.equal(changeState.callCount, 1);
    assert.deepEqual(changeState.args[0][0], {
      sectionA: {
        component: 'services',
        metadata: null
      } });
  });

  it('can cancel the upload', function () {
    var file = {
      name: 'apache2.zip',
      size: '2048'
    };
    var services = {
      toArray: sinon.stub().returns([]) };
    var uploadLocalCharm = sinon.spy();
    var upgradeServiceUsingLocalCharm = sinon.spy();
    var changeState = sinon.spy();
    var shallowRenderer = jsTestUtils.shallowRender(React.createElement(juju.components.LocalInspector, {
      file: file,
      series: series,
      localType: 'new',
      services: services,
      uploadLocalCharm: uploadLocalCharm,
      upgradeServiceUsingLocalCharm: upgradeServiceUsingLocalCharm,
      changeState: changeState }), true);
    var output = shallowRenderer.getRenderOutput();
    output.props.children[2].props.buttons[0].action();
    assert.equal(changeState.callCount, 1);
    assert.deepEqual(changeState.args[0][0], {
      sectionA: {
        component: 'services',
        metadata: null
      } });
  });
});
