/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('machine-view-scale-up', function () {

  juju.components.MachineViewScaleUp = React.createClass({
    displayName: 'MachineViewScaleUp',

    propTypes: {
      addGhostAndEcsUnits: React.PropTypes.func.isRequired,
      services: React.PropTypes.object.isRequired,
      toggleScaleUp: React.PropTypes.func.isRequired
    },

    /**
      Display a list of services.
       @method _generateServices
      @returns {Object} A unit list or onboarding.
    */
    _generateServices: function _generateServices() {
      var components = [];
      var services = this.props.services.toArray();
      services.forEach(function (service) {
        if (service.get('subordinate')) {
          return;
        }
        components.push(React.createElement(
          'li',
          { className: 'machine-view__scale-up-unit',
            key: service.get('id') },
          React.createElement('img', { alt: service.get('name'),
            className: 'machine-view__scale-up-unit-icon',
            src: service.get('icon') }),
          service.get('name'),
          React.createElement('input', {
            className: 'machine-view__scale-up-unit-input',
            placeholder: 'units',
            ref: 'scaleUpUnit-' + service.get('id'),
            type: 'text' })
        ));
      });
      return React.createElement(
        'ul',
        { className: 'machine-view__scale-up-units' },
        components
      );
    },

    /**
      Add units to the services.
       @method _handleAddUnits
      @param {Object} e An event object.
    */
    _handleAddUnits: function _handleAddUnits(e) {
      var _this = this;

      if (e) {
        e.preventDefault();
      }
      var re = /(scaleUpUnit-)(.*)/;
      Object.keys(this.refs).forEach(function (ref) {
        var parts = re.exec(ref);
        if (parts) {
          var service = _this.props.services.getById(parts[2]);
          _this.props.addGhostAndEcsUnits(service, _this.refs[ref].value);
        }
      });
      this.props.toggleScaleUp();
    },

    render: function render() {
      var buttons = [{
        action: this.props.toggleScaleUp,
        title: 'Cancel'
      }, {
        action: this._handleAddUnits,
        title: 'Add units',
        type: 'confirm'
      }];
      return React.createElement(
        'form',
        { className: 'machine-view__scale-up',
          onSubmit: this._handleAddUnits },
        this._generateServices(),
        React.createElement(juju.components.ButtonRow, { buttons: buttons })
      );
    }
  });
}, '0.1.0', {
  requires: ['button-row']
});
