/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('user-profile-list', function () {

  juju.components.UserProfileList = React.createClass({
    displayName: 'UserProfileList',

    propTypes: {
      title: React.PropTypes.string.isRequired,
      data: React.PropTypes.array.isRequired,
      uuidKey: React.PropTypes.string.isRequired,
      switchEnv: React.PropTypes.func.isRequired,
      whitelist: React.PropTypes.array
    },

    /**
      Generates the divs for each key in the row item.
       @method _generateRowItems
      @param {Object} item The object which contains the data to display.
      @param {Boolean} showKeys If you want to show the keys instead of the
        data. This is used to generate the header row.
    */
    _generateRowItems: function _generateRowItems(item, showKeys) {
      var _this = this;

      var items = [];
      Object.keys(item).forEach(function (key) {
        if (_this.props.whitelist) {
          var valid = _this.props.whitelist.some(function (whiteKey) {
            return key === whiteKey;
          });
          if (!valid) {
            return;
          }
        }
        var value = showKeys ? key : item[key];
        var reactKey = item[_this.props.uuidKey] + '-' + key;
        items.push(React.createElement(
          'div',
          {
            className: 'user-profile-list__line-item',
            'data-key': key,
            key: reactKey },
          value
        ));
      });
      return items;
    },

    /**
      Generate the rows from the passed in data.
       @method _generateRows
    */
    _generateRows: function _generateRows() {
      var _this2 = this;

      var rows = [];
      rows.push(this._generateHeaders());
      this.props.data.forEach(function (item) {
        var rowItems = _this2._generateRowItems(item);
        var key = item[_this2.props.uuidKey];
        rows.push(React.createElement(
          'li',
          {
            className: 'user-profile-list__item-row',
            key: key,
            'data-uuid': key,
            onClick: _this2._switchEnv },
          rowItems
        ));
      });
      return rows;
    },

    /**
      Generates the header list item.
       @method _generateHeaders
    */
    _generateHeaders: function _generateHeaders() {
      var header = this._generateRowItems(this.props.data[0], true);
      var key = this.props.title + '-header-row';
      return React.createElement(
        'li',
        {
          className: 'user-profile-list__header-row',
          key: key },
        header
      );
    },

    /**
      Returns the approrpriate body content depending on the existence of
      data. This will display a spinner if there is no data provided. Simply
      re-render the component with data and it will update its UI to the list
      instead.
       @method displayData
    */
    displayData: function displayData() {
      if (!this.props.data || this.props.data.length === 0) {
        return React.createElement(juju.components.Spinner, null);
      }
      // If we have data then dynamically generate the columns and rows
      return React.createElement(
        'ul',
        null,
        this._generateRows()
      );
    },

    /**
      Calls to switch the env to the one the user clicked on.
       @method _switchEnv
      @param {Object} e The click event.
    */
    _switchEnv: function _switchEnv(e) {
      var currentTarget = e.currentTarget;
      this.props.switchEnv(currentTarget.getAttribute('data-uuid'), currentTarget.querySelector('[data-key=name]').innerText);
    },

    render: function render() {
      return React.createElement(
        'div',
        { className: 'user-profile-list' },
        React.createElement(
          'div',
          { className: 'user-profile-list__header' },
          this.props.title,
          React.createElement(
            'span',
            { className: 'user-profile-list__size' },
            ' ',
            ' (',
            this.props.data.length,
            ')'
          )
        ),
        this.displayData()
      );
    }

  });
}, '', {
  requires: ['loading-spinner']
});
